#!/usr/bin/perl
#$Id: UIdata.cgi,v 1.9 2013/08/28 07:02:05 sqko Exp $
use strict;
use warnings;
use RFC::Rig;
use RFC::Main;
use RFC::RFCCGI qw(:cgi :html);
use CGI qw(:standard :netscape);
use Fcntl qw(:DEFAULT :flock);

my $maxdelay = get_cv('main','maximum_time_delay_for_current_values');
$maxdelay = 60 unless $maxdelay;

print header();
my $rig = RFC::Rig->new(param('rig'));
my %setup = ();

### Aquire lock to make suer no other data logging is running ###
my $lockfile = '/tmp/rig_'.$rig->name.'_logfile.lock';
unless (-e $lockfile)
{
    sysopen(FH, $lockfile, O_WRONLY | O_CREAT) or CORE::die "Error: Could not open $lockfile $!";
    chmod 0666, $lockfile;
}
else
{
    sysopen(FH, $lockfile, O_WRONLY | O_CREAT) or CORE::die "Error: Could not open $lockfile $!";
}
flock(FH,LOCK_EX);
### Only use lock to wait for logfile.pl, do not delay unlocking as lockfile.pl may be prevented from starting!
flock(FH,LOCK_UN);
close(FH);

foreach my $name ($rig->get_cv_list('userinterface','temp'))
{
    my $dev = $rig->get_device('templog',$name);
    $setup{'temp'}{$name} = $dev if ($dev);
}

foreach my $name ($rig->get_cv_list('userinterface','gas'))
{		
    my $dev = $rig->get_device('gas',$name);
    $setup{'gas'}{$name} = $dev if ($dev);
}	


foreach my $name ($rig->get_cv_list('userinterface','voltage'))
{		
    my $dev = $rig->get_device('simplechannel',$name);
    $setup{'voltage'}{$name} = $dev if ($dev);
}	

foreach my $name ($rig->get_cv_list('userinterface','current'))
{		
    my $dev = $rig->get_device('simplechannel',$name);
    $setup{'current'}{$name} = $dev if ($dev);
}


my $data = RFC::Cache->new();

foreach my $type (keys %setup)
{
    foreach my $name (keys %{$setup{$type}})
    {
	my ($time,$val) = $rig->get_temp_kanaldata($name);
	$val = $setup{$type}{$name}->read($data) unless (!$time || $time+$maxdelay > time);
	next unless defined($val);
	$val = $val/1000 if ($type eq 'voltage');
	printf("%s\t%.4g\n",$name,$val);
    }
}


